import { useState } from "react";
import { node, number, arrayOf, shape, string, func } from "prop-types";
import Tabs from "@mui/material/Tabs";
import Tab from "@mui/material/Tab";
import Box from "@mui/material/Box";
import { Stack } from "@mui/material";
import { useUniqueKeys } from "../../../hooks/UseUniqueKeys";

const TabContent = ({ children, value, index, ...other }) => {
  return (
    <div
      role="tabpanel"
      hidden={value !== index}
      id={`simple-tabpanel-${index}`}
      aria-labelledby={`simple-tab-${index}`}
      {...other}
    >
      {value === index && <Box>{children}</Box>}
    </div>
  );
};

TabContent.propTypes = {
  children: node,
  index: number.isRequired,
  value: number.isRequired
};

function a11yProps(index) {
  return {
    id: `simple-tab-${index}`,
    "aria-controls": `simple-tabpanel-${index}`
  };
}

/**
 * @typedef TabObject
 * @param {string} label The label rendered for the tab
 * @param {object} content The content to render for this tab
 */

/**
 * @typedef OnTabChangeFcn
 * @function
 * @argument {number} index The index of the tab (starts from 0)
 * @argument {string} label The label of the tab
 */

/**
 * @typedef RenderTabsFcn
 * @function
 * @argument {object[]} tabs that will be rendered
 * @returns {object} React component to render
 */

/**
 * A component that provides conditionally-rendered tabs, for example
 * showing tabs/content based on the permissions of an user.
 * @param {TabObject[]} tabs The tabs to render
 * @param {OnTabChangeFcn} onTabChange Invoked whenever tabs change
 * @param {object} TabsProps props provided to the MUI Tabs component
 * @param {RenderTabsFcn} renderTabs render function that allows customizing
 * where/how the tabs are rendered, for example adding a back arrow icon in front
 * @param {number} initialIndex index to initialize tabs at (starts from zero)
 */
export const TabPanel = ({
  tabs,
  onTabChange,
  TabsProps,
  renderTabs,
  initialIndex
}) => {
  const [tabIndex, setTabIndex] = useState(
    initialIndex && initialIndex >= 0 ? initialIndex : 0
  );

  const tabsKeys = useUniqueKeys(tabs);
  const tabsContentKeys = useUniqueKeys(tabs);

  const handleChange = (event, newValue) => {
    setTabIndex(newValue);
    if (onTabChange) {
      onTabChange(newValue, tabs.at(newValue)?.label); // index, label
    }
  };

  const renderTabElements = () => {
    return (
      <Tabs
        value={tabIndex}
        onChange={handleChange}
        aria-label="basic tabs example"
        {...TabsProps}
      >
        {tabs?.map((tab, i) => (
          <Tab
            key={tabsKeys[i]}
            label={tab.label}
            {...a11yProps(i)}
          />
        ))}
      </Tabs>
    );
  };

  const _renderTabs = () => {
    if (renderTabs) {
      return renderTabs(renderTabElements());
    }
    return renderTabElements();
  };

  return (
    <Stack
      sx={{ width: "100%" }}
      gap={1}
    >
      <Box>{_renderTabs()}</Box>
      {tabs?.map((tab, i) => (
        <TabContent
          key={tabsContentKeys[i]}
          value={tabIndex}
          index={i}
        >
          {tab.content}
        </TabContent>
      ))}
    </Stack>
  );
};
TabPanel.propTypes = {
  tabs: arrayOf(
    shape({
      label: string,
      content: node
    })
  ),
  onTabChange: func,
  renderTabs: func,
  initialIndex: number
};
